@@ -1,4 +1,6 @@
 /*
+ *  arch-tag: Implementation of the internet radio backend database
+ *
  *  Copyright (C) 2002,2003 Colin Walters <walters@gnu.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -15,7 +17,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: rb-iradio-backend.c,v 1.18 2003/08/16 01:58:29 yann Exp $
  */
 
 #include <config.h>
@@ -51,7 +52,6 @@ static void station_changed_cb (RBNode *node, RBNode *parent, guint propid, RBIR
 
 static RBNode * rb_iradio_backend_lookup_station_by_location (RBIRadioBackend *backend,
 							      const char *uri);
-static gboolean rb_iradio_backend_periodic_save (RBIRadioBackend *backend);
 static void finalize_node (RBNode *node);
 static void restore_node (RBNode *node);
 static void sync_sort_keys (RBNode *node);
@@ -70,8 +70,6 @@ struct RBIRadioBackendPrivate
 	GStaticRWLock *genre_hash_lock;
 
 	char *xml_file;
-
-	guint idle_save_id;
 };
 
 enum
@@ -208,8 +206,6 @@ rb_iradio_backend_init (RBIRadioBackend *backend)
 	rb_node_add_child (backend->priv->all_genres,
 			   backend->priv->all_stations);
 
-	backend->priv->idle_save_id = g_idle_add ((GSourceFunc) rb_iradio_backend_periodic_save,
-						  backend);
 }
 
 static void
@@ -224,8 +220,6 @@ rb_iradio_backend_finalize (GObject *object)
 
 	g_return_if_fail (backend->priv != NULL);
 
-	g_source_remove (backend->priv->idle_save_id);
-
 	rb_iradio_backend_save (backend);
 
 	rb_node_unref (backend->priv->all_stations);
@@ -406,17 +400,6 @@ void rb_iradio_backend_load (RBIRadioBackend *backend)
 	load_initial (backend);
 }
 
-static gboolean
-rb_iradio_backend_periodic_save (RBIRadioBackend *backend)
-{
-	rb_debug ("doing periodic save");
-	rb_iradio_backend_save (backend);
-	backend->priv->idle_save_id = g_timeout_add (60000 + (g_random_int_range (0, 15) * 1000),
-						     (GSourceFunc) rb_iradio_backend_periodic_save,
-						     backend);
-	return FALSE;
-}
-
 static void
 rb_iradio_backend_save (RBIRadioBackend *backend)
 {
