@@ -323,8 +323,8 @@ rb_marshal_BOOLEAN__POINTER (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* VOID:FLOAT (./rb-marshal.list:7) */
-#define rb_marshal_VOID__FLOAT	g_cclosure_marshal_VOID__FLOAT
+/* VOID:DOUBLE (./rb-marshal.list:7) */
+#define rb_marshal_VOID__DOUBLE	g_cclosure_marshal_VOID__DOUBLE
 
 G_END_DECLS
 
