@@ -1,4 +1,6 @@
-/* 
+/*
+ *  arch-tag: Implementation of main song information database object
+ *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
  *  Copyright (C) 2003 Colin Walters <walters@debian.org>
  *
@@ -16,7 +18,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: rb-library.c,v 1.69 2003/08/19 02:56:42 cwalters Exp $
  */
 
 #include <config.h>
@@ -56,7 +57,6 @@ static void sync_node (RBNode *node,
 		       GError **error);
 static void finalize_node (RBNode *node);
 static void restore_node (RBNode *node);
-static gboolean rb_library_periodic_save (RBLibrary *library);
 static void sync_sort_keys (RBNode *node);
 static void walker_thread_done_cb (RBLibraryWalkerThread *thread, RBLibrary *library);
 static char *get_status_normal (RBLibrary *library, RBNode *root, RBNodeFilter *filter);
@@ -109,8 +109,6 @@ struct RBLibraryPrivate
 
 	guint refresh_count;
 
-	guint idle_save_id;
-
 	gboolean status_poll_queued;
 	guint status_poll_id;
 
@@ -197,9 +195,9 @@ rb_library_class_init (RBLibraryClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (RBLibraryClass, progress),
 			      NULL, NULL,
-			      rb_marshal_VOID__FLOAT,
+			      rb_marshal_VOID__DOUBLE,
 			      G_TYPE_NONE,
-			      1, G_TYPE_FLOAT);
+			      1, G_TYPE_DOUBLE);
 }
 
 static void
@@ -270,8 +268,6 @@ rb_library_release_brakes (RBLibrary *library)
 	g_signal_connect (G_OBJECT (library->priv->main_thread), "error",
 			  G_CALLBACK (rb_library_pass_on_error), library);
 
-	library->priv->idle_save_id = g_idle_add ((GSourceFunc) rb_library_periodic_save,
-						  library);
 }
 
 static gboolean
@@ -328,8 +324,6 @@ rb_library_finalize (GObject *object)
 
 	g_return_if_fail (library->priv != NULL);
 
-	g_source_remove (library->priv->idle_save_id);
-
 	rb_debug ("library: finalizing");
 
 	g_mutex_free (library->priv->walker_mutex);
@@ -409,14 +403,14 @@ signal_progress_changed (RBLibrary *library)
 	if (!queue_is_empty (library->priv->add_queue)) {
 		g_signal_emit (G_OBJECT (library), rb_library_signals[PROGRESS], 0, -1.0);
 	} else if (library->priv->state == LIBRARY_STATE_INITIAL_REFRESH) {
-		float total;
-		float refresh_count;
+		double total;
+		double refresh_count;
 
 		g_mutex_lock (library->priv->lock);
-		refresh_count = (float) library->priv->refresh_count;
+		refresh_count = (double) library->priv->refresh_count;
 		g_mutex_unlock (library->priv->lock);
 
-		total = (float) rb_node_get_n_children (library->priv->all_songs);
+		total = (double) rb_node_get_n_children (library->priv->all_songs);
 		
 		g_signal_emit (G_OBJECT (library), rb_library_signals[PROGRESS], 0,
 			       refresh_count / total);
@@ -484,8 +478,8 @@ rb_library_add_uri_sync (RBLibrary *library, const char *uri, GError **error)
 
 signal:
 	if (!library->priv->in_shutdown) {
-		signal_status_changed (library);
 		signal_progress_changed (library);
+		signal_status_changed (library);
 	}
 }
 
@@ -523,8 +517,8 @@ rb_library_update_uri (RBLibrary *library, const char *uri, GError **error)
 	rb_file_monitor_add (rb_file_monitor_get (), uri);
 	ret = TRUE;
 out:
-	signal_status_changed (library);
 	signal_progress_changed (library);
+	signal_status_changed (library);
 	return ret;
 }
 
@@ -790,20 +784,6 @@ rb_library_create_skels (RBLibrary *library)
 	rb_debug ("Done creating skels");
 }
 
-static gboolean
-rb_library_periodic_save (RBLibrary *library)
-{
-	if (rb_library_is_idle (library)) {
-		rb_debug ("doing periodic save");
-		rb_library_save (library);
-	} else {
-		rb_debug ("library is busy, skipping periodic save");
-	}
-	library->priv->idle_save_id = g_timeout_add (60000 + (g_random_int_range (0, 15) * 1000),
-						     (GSourceFunc) rb_library_periodic_save, library);
-	return FALSE;
-}
-
 static void
 rb_library_save (RBLibrary *library)
 {
@@ -1561,7 +1541,7 @@ rb_library_compute_status (RBLibrary *library, RBNode *root, RBNodeFilter *filte
 	gboolean adding_songs = !queue_is_empty (library->priv->add_queue);
 
 	if (adding_songs)
-		ret = g_strdup_printf (_("<b>Loading songs...</b>"));
+		ret = g_strdup_printf ("<b>%s</b>", _("Loading songs..."));
 	else if (library->priv->state == LIBRARY_STATE_INITIAL_REFRESH) {
 		guint refresh_count;
 		
@@ -1574,7 +1554,7 @@ rb_library_compute_status (RBLibrary *library, RBNode *root, RBNodeFilter *filte
 			library->priv->status_poll_queued = TRUE;
 		}
 	
-		ret = g_strdup_printf (_("<b>Refreshing songs...</b>"));
+		ret = g_strdup_printf ("<b>%s</b>", _("Refreshing songs..."));
 	}
 
 	if (adding_songs || library->priv->state == LIBRARY_STATE_INITIAL_REFRESH) {
@@ -1603,6 +1583,8 @@ poll_status_update (gpointer data)
 		if (library->priv->state == LIBRARY_STATE_INITIAL_REFRESH)
 			library->priv->state = LIBRARY_STATE_NONE;
 
+		g_get_current_time (&library->priv->mod_time);
+
 		library->priv->status_poll_queued = FALSE;
 
 		signal_status_changed (library);
