@@ -1,4 +1,6 @@
 /*
+ *  arch-tag: Header for main song information database object
+ *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
  *  Copyright (C) 2003 Colin Walters <walters@rhythmbox.org>
  *
@@ -16,7 +18,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: rb-library.h,v 1.27 2003/08/19 02:56:42 cwalters Exp $
  */
 
 #ifndef __RB_LIBRARY_H
@@ -51,7 +52,7 @@ typedef struct
 
 	/* signals */
 	void	(*error)	(const char *uri, const char *msg);
-	void	(*progress)(float val);
+	void	(*progress)(double val);
 	void	(*status_changed)();
 } RBLibraryClass;
 
