@@ -1,4 +1,6 @@
-/*
+/* 
+ *  arch-tag: Header for object implementing main playback logic
+ *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -15,7 +17,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: rb-shell-player.h,v 1.13 2003/08/07 17:41:07 cwalters Exp $
  */
 
 #include <gtk/gtkhbox.h>
@@ -66,6 +67,8 @@ RBSource *		rb_shell_player_get_source	(RBShellPlayer *shell_player);
 
 void			rb_shell_player_jump_to_current (RBShellPlayer *player);
 
+void			rb_shell_player_playpause	(RBShellPlayer *player);
+
 void			rb_shell_player_stop		(RBShellPlayer *shell_player);
 
 void			rb_shell_player_set_shuffle	(RBShellPlayer *shell_player,
@@ -77,12 +80,13 @@ gboolean		rb_shell_player_get_playing	(RBShellPlayer *shell_player);
 
 const char *		rb_shell_player_get_playing_path(RBShellPlayer *shell_player);
 
+void			rb_shell_player_sync_buttons (RBShellPlayer *player);
+
+
 #ifdef HAVE_ACME
 gboolean		rb_shell_player_handle_key	(RBShellPlayer *player, guint keyval);
 #endif
 
-gboolean		rb_shell_volume_scroll 		(GtkWidget *widget, GdkEvent *event, gpointer data);
-
 G_END_DECLS
 
 #endif /* __RB_SHELL_PLAYER_H */
