@@ -1,4 +1,6 @@
-/* 
+/*
+ *  arch-tag: Implementation of status display widget
+ *
  *  Copyright (C) 2003 Colin Walters <walters@debian.org>
  *  Copyright (C) 2003 Jorn Baayen <jorn@nl.linux.org>
  *
@@ -16,7 +18,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: rb-statusbar.c,v 1.8 2003/08/19 02:56:40 cwalters Exp $
  */
 
 #include <gtk/gtk.h>
@@ -49,7 +50,6 @@ static void rb_statusbar_get_property (GObject *object,
 static void rb_statusbar_sync_with_source (RBStatusbar *statusbar);
 static void rb_statusbar_status_changed_cb (RBSource *source,
 					    RBStatusbar *statusbar);
-static void rb_statusbar_sync_state (RBStatusbar *statusbar);
 static void rb_statusbar_state_changed_cb (GConfClient *client,
 					   guint cnxn_id,
 					   GConfEntry *entry,
@@ -312,7 +312,7 @@ status_tick_cb (GtkProgressBar *progress)
 }
 
 void
-rb_statusbar_set_progress (RBStatusbar *statusbar, float progress)
+rb_statusbar_set_progress (RBStatusbar *statusbar, double progress)
 {
 	if (progress > 1.0)
 		progress = 1.0;
@@ -363,7 +363,7 @@ rb_statusbar_status_changed_cb (RBSource *source,
 	rb_statusbar_sync_with_source (statusbar);
 }
 
-static void
+void
 rb_statusbar_sync_state (RBStatusbar *statusbar)
 {
 	gboolean hidden;
