@@ -1,4 +1,6 @@
 /*
+ *  arch-tag: Header for status display widget
+ *
  *  Copyright (C) 2003 Colin Walters <walters@verbum.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -15,7 +17,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: rb-statusbar.h,v 1.3 2003/08/19 02:56:40 cwalters Exp $
  */
 
 #include <gtk/gtkhbox.h>
@@ -56,7 +57,8 @@ RBStatusbar *		rb_statusbar_new	(BonoboUIComponent *component);
 void			rb_statusbar_set_source	(RBStatusbar *bar,
 						 RBSource *player);
 
-void			rb_statusbar_set_progress(RBStatusbar *bar, float progress);
+void			rb_statusbar_set_progress(RBStatusbar *bar, double progress);
+void			rb_statusbar_sync_state (RBStatusbar *statusbar);
 
 G_END_DECLS
 
